<?php

namespace OCA\GrueneGroupfolderManagement\Controller;

use Exception;
use OCP\AppFramework\Http\JSONResponse;
use OCP\AppFramework\Http;
use OCP\AppFramework\Http\Attribute\NoAdminRequired;
use OCP\App\IAppManager;
use Psr\Container\ContainerInterface;

use OCA\GrueneGroupfolderManagement\Exceptions\NotFoundException;
use OCA\GrueneGroupfolderManagement\Manager\GroupFolderManager;
use OCA\GrueneGroupfolderManagement\Manager\GroupManager;

use OCA\GroupfolderFilesystemSnapshots\Manager\SnapshotManager;
use OCA\GroupfolderFilesystemSnapshots\Service\DiffTaskService;
use OCA\GroupfolderFilesystemSnapshots\Service\DiffTaskResultService;

class DiffResultController extends BaseController {

	private bool $snapshotIntegrationEnabled;
	private SnapshotManager $snapshotManager;
	private DiffTaskService $diffTaskService;
	private DiffTaskResultService $diffTaskResultService;

	public function __construct(
		private IAppManager $appManager,
		private ContainerInterface $container,
		
        private GroupFolderManager $groupFolderManager,
        private GroupManager $groupManager,
        private ?string $userId,
	) {
		parent::__construct();

		$this->snapshotIntegrationEnabled = $this->appManager->isEnabledForUser("groupfolder_filesystem_snapshots");

		if($this->snapshotIntegrationEnabled) {
			$this->snapshotManager = $this->container->get(SnapshotManager::class);
			$this->diffTaskService = $this->container->get(DiffTaskService::class);
			$this->diffTaskResultService = $this->container->get(DiffTaskResultService::class);
		}
	}

    use Errors;

	private function findTaskResultIfAccessAllowed(string $snapshotId, int $diffTaskId, int $diffTaskResultId, array $attributes) {
        $taskResult = $this->diffTaskResultService->find($diffTaskResultId);

		$task = $this->diffTaskService->find($taskResult->getTaskId(), $this->userId);
		$this->denyAccessUnlessGranted($attributes, $task);

		if($taskResult->getTaskId() !== $diffTaskId || $task->getSnapshotId() !== $snapshotId) {
			throw new NotFoundException(SnapShotManager::class, ['snapshotId' => $snapshotId, 'diffTaskId' => $diffTaskId, "diffTaskResultId" => $diffTaskResultId]);
        }

		return $taskResult;
	}

    #[NoAdminRequired]
    public function show(string $snapshotId, int $diffTaskId, int $diffTaskResultId): JSONResponse {
		return new JSONResponse($this->findTaskResultIfAccessAllowed($snapshotId, $diffTaskId, $diffTaskResultId, ['READ']));
    }

    #[NoAdminRequired]
	public function revert(string $snapshotId, int $diffTaskId, int $diffTaskResultId): JSONResponse {
		$taskResult = $this->findTaskResultIfAccessAllowed($snapshotId, $diffTaskId, $diffTaskResultId, ['REVERT']);
		return new JSONResponse($this->diffTaskResultService->revert($taskResult->getId(), $this->userId));
    }
}