<?php

namespace OCA\GrueneGroupfolderManagement\Controller;

use Exception;
use OCP\AppFramework\Http;
use OCP\AppFramework\Http\JSONResponse;
use OCP\AppFramework\Http\Attribute\NoAdminRequired;
use OCP\App\IAppManager;
use Psr\Container\ContainerInterface;

use OCA\GroupfolderFilesystemSnapshots\Manager\SnapshotManager;
use OCA\GrueneGroupfolderManagement\Exceptions\NotFoundException;

class SnapshotController extends BaseController {
	private bool $snapshotIntegrationEnabled;

	private SnapshotManager $snapshotManager;

	public function __construct(
		private IAppManager $appManager,
		private ContainerInterface $container,
        private ?string $userId,
	) {
		parent::__construct();

		$this->snapshotIntegrationEnabled = $this->appManager->isEnabledForUser("groupfolder_filesystem_snapshots");

		if($this->snapshotIntegrationEnabled) {
			$this->snapshotManager = $this->container->get(SnapshotManager::class);
		}
	}

    use Errors;

    #[NoAdminRequired]
    public function index(): JSONResponse {
		if($this->snapshotIntegrationEnabled) {
        	return new JSONResponse(iterator_to_array($this->snapshotManager->getAll()));
		} else {
			return new JSONResponse([
				"status" => "error",
				"errorMessage" => "Snapshot integration not active",
			], Http::STATUS_NOT_FOUND);
		}
    }

    #[NoAdminRequired]
    public function show(string $snapshotId): JSONResponse {
		if($this->snapshotIntegrationEnabled) {
			if(!$this->snapshotManager->snapshotExists($snapshotId)) {
				throw new NotFoundException(SnapShotManager::class, ['id' => $snapshotId]);
			}

			return new JSONResponse($this->snapshotManager->get($snapshotId));
		} else {
			return new JSONResponse([
				"status" => "error",
				"errorMessage" => "Snapshot integration not active",
			], Http::STATUS_NOT_FOUND);
		}
    }
}