<?php

namespace OCA\GrueneGroupfolderManagement\Security;

use OCA\GroupfolderFilesystemSnapshots\Db\DiffTask;
use OCA\GrueneGroupfolderManagement\Db\GroupMapper;
use OCA\GrueneGroupfolderManagement\Security\AuthorizationService;
use OCA\GrueneGroupfolderManagement\Manager\GroupFolderManager;
use OCP\IUser;

class DiffTaskVoter extends Voter {
	public function __construct(
		private GroupMapper $groupMapper,
		private GroupFolderManager $groupFolderManager,
	) {
	}
	protected function supports(string $attribute, mixed $subject): bool {
		return $subject instanceof DiffTask || $subject === DiffTask::class;
	}

	protected function voteOnAttribute(string $attribute, mixed $subject, ?IUser $user): bool {
		if (!$user) {
			return false;
		}

		/** @var DiffTask */
		$diffTask = $subject;
		return match ($attribute) {
			'READ' => $this->isGranted($user, $diffTask),
			'REVERT' => $this->isGranted($user, $diffTask),
			default => throw new \LogicException('This code should not be reached!')
		};
	}

	protected function isGranted(IUser $user, DiffTask $diffTask) : bool {
		// only allow access to diff tasks the user created
		if($diffTask->getUserId() !== $user->getUID()) {
			return false;
		}

		// only allow access if the user has not lost access to the group since creating the diff task
		$groupfolder = $this->groupFolderManager->getGroupFolderByFolderId($diffTask->getGroupfolderId());
		$groupfolderDirectory = $this->groupFolderManager->getGroupFolder($diffTask->getGroupfolderId(), False);
		$grueneFolderId = $groupfolderDirectory->get($diffTask->getRelativePath())->getId();
		$group = $this->groupMapper->getGroupByFolderId($groupfolder['organization_key'], $grueneFolderId);

		$authorizationService = \OC::$server->get(AuthorizationService::class);

		if (!$authorizationService->isGranted(['UPDATE'], $group)) {
			return false;
		}

		return true;
	}
}
