<?php

namespace OCA\GrueneGroupfolderManagement\Security;

use OCA\GrueneGroupfolderManagement\Db\Group;
use OCA\GrueneGroupfolderManagement\Manager\SherpaManager;
use OCA\GrueneGroupfolderManagement\Model\Division;
use OCP\IUser;
use OCP\IGroupManager;

class GroupVoter extends Voter {
	public function __construct(
		private SherpaManager $sherpaManager,
		private IGroupManager $groupManager,
	) {
	}
	protected function supports(string $attribute, mixed $subject): bool {
		return $subject instanceof Group || $subject === Group::class;
	}


	protected function voteOnAttribute(string $attribute, mixed $subject, ?IUser $user): bool {
		// _dlog($attribute, $subject);

		if (!$user) {
			return false;
		}

		// if ($subject instanceof Group) {

		// }

		/** @var Group */
		$group = $subject;
		return match ($attribute) {
			'CREATE' => $this->isGranted($user, $group),
			'READ' => $this->isGranted($user, $group),
			'UPDATE' => $this->isGranted($user, $group),
			'DELETE' => $this->isGranted($user, $group),
			default => throw new \LogicException('This code should not be reached!')
		};
	}

	/**
	 * @param IUser $user
	 * @return Division[]
	 */
	private function getAdministerableDivisions(IUser $user) : \Traversable {
		foreach ($this->sherpaManager->filterDivisionRoles($user, 'TECHNICAL_WOLKE') as $divisionGroup) {
			yield $divisionGroup->getDivision();
		}

		yield from [];
	}

	/**
	 * @param IUser $user
	 * @param Group $group
	 * @return bool
	 */
	private function isGroupAdmin(IUser $user, Group $group) : bool {
		$division = $group->getDivision();
		$userGroups = $this->groupManager->getUserGroupIds($user);
		$adminPattern = sprintf("/^GROUP_%d_%d_ADMINS$/", $division->getId(), $group->getId());

		return !empty(preg_grep($adminPattern, $userGroups));
	}

	protected function isGranted(IUser $user, Group $group) : bool {
		if($this->isGroupAdmin($user, $group)) {
			return true;
		}

		// TODO better performance
		$division = $group->getDivision();
		foreach ($this->getAdministerableDivisions($user) as $granted) {
			if ($division->getOrganizationalUuid() == $granted->getOrganizationalUuid()) {
				return true;
			}
		}

		return false;
	}
}
