<?php

declare(strict_types=1);

namespace OCA\GrueneGroupfolderManagement\Service;

use OCA\GrueneGroupfolderManagement\AppInfo\Application;
use OCP\IConfig;

class SettingsService {

	private static array $VALID_APP_SETTINGS = ["group_filter_pattern", "role_filter_pattern", "default_quota", "share_user", "share_group", "api_server", "api_user", "api_password"];

	private static array $APP_SETTINGS_DEFAULTS = [
		"group_filter_pattern" => "^SAML_([0-9]+)",
		"role_filter_pattern" => '^SAML_[0-9]+_([0-9]+)',
		"default_quota" => 5368709120,
		"api_server" => 'https://api.netzbegruenung.de/',
	];
	
	public function __construct(private IConfig $config) {
	}

	public function getAppValues(): array {
		$result = [];
		foreach(self::$VALID_APP_SETTINGS as $key) {
			$result[$key] = $this->config->getAppValue(Application::APP_ID, $key, self::$APP_SETTINGS_DEFAULTS[$key] ?? "");
		}
		return $result;
	}

	public function getAppValue(string $key): string {
		if(in_array($key, self::$VALID_APP_SETTINGS)) {
			return $this->config->getAppValue(Application::APP_ID, $key, self::$APP_SETTINGS_DEFAULTS[$key] ?? "");
		}
	}
    public function setAppValue(string $key, string $value): string {
		if(in_array($key, self::$VALID_APP_SETTINGS)) {
			if($value !== '') {
				$this->config->setAppValue(Application::APP_ID, $key, $value);
			} else {
				$this->config->deleteAppValue(Application::APP_ID, $key);
			}

			return $value;
		}
    }
}